#include "database.h"
#include "QFile"
#include "QTextStream"
#include "QDate"

//This function creates a new Database object and attempts to create a
//database connection using the global variables dbuser, dbpass, and dbhost.
//Those variables must be set before a database is created.
//
//This contructor can throw an int(1) if there is a failure to connect.
//Please be aware of this.
Database::Database(QString connName, QString dbname) throw(int){
    extern QString dbuser;
    extern QString dbpass;
    extern QString dbhost;
    int dbport = 3306;

    //This is here to avoid a complaint by qt, if there is a problem
    //while connecting and qt does not remove the database, this will
    //take care of it
    QSqlDatabase::removeDatabase(connName);

    dbs = new QSqlDatabase(QSqlDatabase::addDatabase("QMYSQL",connName));

    dbs->setHostName(dbhost);
    dbs->setDatabaseName(dbname);
    dbs->setUserName(dbuser);
    dbs->setPassword(dbpass);
    dbs->setPort(dbport);

    if (!dbs->open()){
        qDebug() << dbs->lastError() << endl;
        delete dbs;
        QSqlDatabase::removeDatabase(connName);
        throw 1;
    }

}

//This hashes the passwords given for user addition or login
QByteArray Database::hashPassword(QString user, QString pass){
    int half = pass.length()/2;
    //I salt the username into the middle of the password
    QString salted1 = pass.mid(0,half) + user + pass.mid(half);
    QCryptographicHash hash1(QCryptographicHash::Md5);
    //addData required a QByteArray, toAscii does that
    hash1.addData(salted1.toAscii());
    QCryptographicHash hash2(QCryptographicHash::Sha1);
    //result returns a QByteArray
    hash2.addData(hash1.result());
    return hash2.result();
}

//This discovers the mysql error number from a database error and
//converts it into an error code for the system.
int Database::getErrorNumber(QSqlError err)
{
    QFile debug("database_error.log");
    if (debug.open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text))
    {
        QTextStream out(&debug);
        QDateTime current = QDateTime::currentDateTime();
        out << current.toString(Qt::ISODate) << " " << err.number() <<", "<<err.text() << "\n";
        debug.close();
    }
    //qDebug() << err << endl;
    int mysqlErr = err.number();
    if (mysqlErr == -1)
    {
        //unknown error
        return 100;
    }
    else if (mysqlErr == 2003)
    {
        //initial connection failed
        return 1;
    }
    else if (mysqlErr == 2006 || mysqlErr == 2013)
    {
        //lost connection
        return 2;
    }
    else if (mysqlErr == 1050)
    {
        //table already exists
        return 3;
    }
    else if (mysqlErr == 1062)
    {
        //duplicate primary key
        return 4;
    }
    else if (mysqlErr == 1235)
    {
        //duplicate trigger, ignore
        return 5;
    }
    else if (mysqlErr == 1146)
    {
        //category doesn't exist
        return 6;
    }
    else if (mysqlErr == 1452)
    {
        //foreign constraint failed, reference doesn't exist
        return 7;
    }
    else if (mysqlErr == 1360)
    {
        //trigger doesn't exist to delete
        return 8;
    }
    else
    {
        //unhandled error
        return -100;
    }
}

//This attempts to begin a transaction on the database
bool Database::transaction(){
    return this->dbs->transaction();
}

//This attempts to commit a database transaction
bool Database::commit(){
    return this->dbs->commit();
}

//This attempts to rollback a database transaction
bool Database::rollback(){
    return this->dbs->rollback();
}

Database::~Database(){
    qDebug() << "destruct" << endl;
    delete dbs;
}
///////////////////////////////////////////////////////////////////
//Tables
////////////////////////////////////////////////////////////////////

//This will attempt to create the users table and add a first user
//with the given username and already hashed password.
//
//If the database connection has failed, this will throw an integer
//If the table and user already exist, this will throw an integer
//indicating duplicate primary key entry
//
//NOTE: This function executes multiple sql statements, if a transaction is needed
// the caller must perform it.
bool Database::setInitialUser(QString username,QByteArray password){
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare(QString("create table IF NOT EXISTS users( username varchar(25) primary key, password varchar(30)) Engine = Innodb;"));
    if (!query.exec()){
        throw getErrorNumber(query.lastError());
    }
    query.prepare("insert into users (username, password) values (?, ?)");
    query.bindValue(0,username);
    query.bindValue(1,password);
    if (query.exec()){
        verdict = true;
    }
    else {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

//This will attempt to add the tables and triggers necessary to install the system
//
//Any mysql error generated while adding the tables will cause a system error code to be thrown.
//While adding the triggers, any mysql error other than "duplicate trigger" will cause a thrown integer.
//
//NOTE: This function executes multiple sql statements, if a transaction is needed
// the caller must perform it.
bool Database::setupTables(){
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare(QString("create table IF NOT EXISTS people( Tnumber char(9) primary key DEFAULT 'T00000000', LastName varchar(15) DEFAULT 'LastName', FirstName varchar(15) DEFAULT 'FirstName', strAddr varchar(75) DEFAULT 'No address given.', HPhone varchar(20) DEFAULT 'No home phone given.', WPhone varchar(20) DEFAULT 'No work phone given.', CPhone varchar(20) DEFAULT 'No cell phone given.', email varchar(25) DEFAULT 'No e-mail address given.') Engine = Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 2" << endl;
        throw getErrorNumber(query.lastError());
    }
    query.prepare(QString("create table IF NOT EXISTS pkeys( TagNum int primary key, KeyName varchar(10) DEFAULT 'KeyName', Total int DEFAULT 1, InInv int DEFAULT 1) Engine = Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 3" << endl;
        qDebug() << dbs->lastError() << endl;

        throw getErrorNumber(query.lastError());
    }
    query.prepare(QString("create table IF NOT EXISTS p_k (TransactionID int primary key AUTO_INCREMENT, Tnumber char(9) DEFAULT 'T00000000', TagNum int DEFAULT 0, NumOut int DEFAULT 1, DateOut date, DateDue date DEFAULT null, FOREIGN KEY (Tnumber) REFERENCES people(Tnumber), FOREIGN KEY (TagNum) REFERENCES pkeys(TagNum) ON DELETE cascade) Engine=Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 4" << endl;

        throw getErrorNumber(query.lastError());
    }
    query.prepare(QString("create table IF NOT EXISTS groups ( Gid int primary key auto_increment, GroupName varchar(40) DEFAULT 'GroupName' UNIQUE) Engine = Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 5" << endl;

        throw getErrorNumber(query.lastError());
    }
    query.prepare(QString("create table IF NOT EXISTS p_g ( Tnumber char(9) DEFAULT 'T00000000', Gid int DEFAULT 0, PRIMARY KEY (Tnumber, Gid), FOREIGN KEY (Tnumber) REFERENCES people(Tnumber) ON DELETE cascade, FOREIGN KEY (Gid) REFERENCES groups(Gid) ON DELETE cascade) Engine = Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 6" << endl;

        throw getErrorNumber(query.lastError());
    }
    query.prepare(QString("create table IF NOT EXISTS rooms ( RoomId int primary key auto_increment, RoomName varchar(10) DEFAULT 'RoomName' UNIQUE ) Engine = Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 7" << endl;

        throw getErrorNumber(query.lastError());
    }
    query.prepare(QString("create table IF NOT EXISTS k_r ( TagNum int DEFAULT 0, RoomId int DEFAULT 0, PRIMARY KEY (TagNum, RoomId), FOREIGN KEY (TagNum) REFERENCES pkeys(TagNum) ON DELETE cascade, FOREIGN KEY (RoomId) REFERENCES rooms(RoomID)ON DELETE cascade ) Engine = Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 8" << endl;

        throw getErrorNumber(query.lastError());
    }
    query.prepare(QString("create table IF NOT EXISTS g_r ( Gid int DEFAULT 0, RoomId int DEFAULT 0, PRIMARY KEY (Gid, RoomId), FOREIGN KEY (Gid) REFERENCES groups(Gid) ON DELETE cascade, FOREIGN KEY (RoomId) REFERENCES rooms(RoomID) ON DELETE cascade ) Engine = Innodb;"));
    if (!query.exec()){
        qDebug() << "Table 9" << endl;

        throw getErrorNumber(query.lastError());
    }

    query.exec("set DELIMITER = \\");
    query.prepare(QString("create trigger now_date_pk BEFORE INSERT ON p_k FOR EACH ROW set NEW.DateOut=now();"));
    if (!query.exec()){
        qDebug() << "Trigger 1" << endl;
        int err = getErrorNumber(query.lastError());
        if (err != 5){
            throw err;
        }
    }

    query.prepare(QString("create trigger checkout_keys AFTER INSERT ON p_k FOR EACH ROW BEGIN DECLARE num INT; set num = NEW.NumOut; update pkeys set InInv = InInv - num where TagNum = new.TagNum; END"));
    if (!query.exec()){
        qDebug() << "Trigger 2" << endl;

        int err = getErrorNumber(query.lastError());
        if (err != 5){
            throw err;
        }
    }

    query.prepare(QString("create trigger change_keys AFTER UPDATE ON p_k FOR EACH ROW BEGIN DECLARE num INT; set num = OLD.NumOut - NEW.NumOut; update pkeys set InInv = InInv + num where TagNum = new.TagNum; END"));
    if (!query.exec()){
        qDebug() << "Trigger 3" << endl;

        int err = getErrorNumber(query.lastError());
        if (err != 5){
            throw err;
        }
    }
    query.prepare(QString("create trigger checkin_keys AFTER DELETE ON p_k FOR EACH ROW BEGIN DECLARE num INT; set num = OLD.NumOut; update pkeys set InInv = InInv + num where TagNum = OLD.TagNum; END"));
    if (!query.exec()){
        qDebug() << "Trigger 4" << endl;

        int err = getErrorNumber(query.lastError());
        if (err != 5){
            throw err;
        }
    }
    query.exec("set DELIMITER = ;");

    verdict = true;
    return verdict;
}

//////////////////////////////////////////////////////////////////////
//End Tables
/////////////////////////////////////////////////////////////////////


