#ifndef DATABASE_H
#define DATABASE_H

#include <QCryptographicHash>
#include <QDebug>
#include <QtSql>
#include <QList>

class person;
class group;
class inventory;
class key;
class category_item;
class room;
class dueDate;


// This class provides a way of accessing the mysql database
// all methods will be commented on when they are defined
class Database{

public:

    //This enum handles the general ways in which a person can be queried for
    enum PersonQueryMode {ALL = -1, CUSTOM = -2, WITH_KEYS = -3, IN_GROUPS = -4, WITH_OTHER_ITEMS = -5};
    //This enum handles how to search for people with access to a given room
    enum RoomQueryMode {BOTH = -1, KEYS_ONLY = -2, GROUPS_ONLY = -3};
    //This enum handles how to add or remove rooms to/from groups or keys
    enum RoomAssociationMode {ADD_ROOMS = -1, REMOVE_ROOMS = -2, TO_KEY = 1, TO_GROUP = 2};

    Database(QString connName, QString dbname = "cscinventory") throw(int);
    ~Database();
    //General, in database.cpp
    static QByteArray hashPassword(QString, QString);
    static int getErrorNumber(QSqlError);
    bool transaction();
    bool commit();
    bool rollback();
    //

    // Setup, in database.cpp
    bool setInitialUser(QString, QByteArray);
    bool setupTables();
    //

    // Users, in database_users.cpp
    bool login(QString, QByteArray);
    bool addUser(QString, QByteArray);
    bool resetPass(QString, QByteArray);
    QStringList getUsers();
    bool removeUser(QString);
    //

    // People, in database_people.cpp
    bool addPerson(person* to_add);
    bool updatePerson(person*);
    bool removePerson(person*);
    QList<person*>* getPeople(Database::PersonQueryMode mode,person* to_search = NULL,QStringList* categories = NULL);
    QList<group*>* getPersonGroups(person*);
    QList<inventory*>* getPersonInv(person*,QStringList);
    bool removePersonFromGroup(group*,person*);
    void getPersonInfo(person*);
    //

    // Rooms, in database_rooms.cpp
    void addRoom(room*);
    bool removeRoom(room*);
    void doesRoomExist(room*);
    QList<room*>* getRooms();
    QList<group*>* getRoomGroups(room*);
    QList<key*>* getRoomKeys(room*);
    QList<person*>* getPeopleWithAccessToRooms(Database::RoomQueryMode,room*);
    //

    // Groups, in database_groups.cpp
    void addGroup(group*);
    bool removeGroup(group*);
    void doesGroupExist(group*);
    bool addPersonToGroup(group*,person*);
    bool removeAllPeopleFromGroup(group*);
    QList<group*>* getGroups();
    QList<person*>* getGroupPeople(group*);
    QList<room*>* getGroupRooms(group*);
    //

    bool modifyRoomList(Database::RoomAssociationMode table, Database::RoomAssociationMode function, int Gid, QList<room*>* rooms);

    // Keys, in database_keys.cpp
    QList<key*>* getKeys(key*, QList<room*>*);
    QList<key*>* getAllKeys();
    QList<room*>* getKeyRooms(key*);
    //

    // Other Inventory, in database_otherinv.cpp
    QStringList getCategory();
    bool addCategory(QString);
    bool removeCategory(QString);
    bool addInventoryItem(inventory*);
    bool removeInventoryItem(inventory*);
    bool CheckOutInventory(inventory*,person*);
    bool CheckInInventory(inventory*);
    bool updateItemQuantity(inventory*);
    void doesCheckOutExist(inventory*,person*);
    void getCheckOutInfo(inventory*,person*);
    QList<category_item*>* getItems(category_item*);
    void getInventoryInfo(inventory*);
    QList<person*>* getPeopleWithInventoryCheckedOut(inventory*);
    //

    //Calendar, in database_calendar.cpp
    QList<dueDate*>* getDueWithinMonth(QDate,QStringList);

private:
    QSqlDatabase *dbs;

};

#endif // DATABASE_H
