#include "database.h"
#include "group.h"
#include "room.h"
#include "person.h"

//////////////////////////////////////////////////////////////////////////
// Groups
//////////////////////////////////////////////////////////////////////////

//This takes a pointer to a group object and REQUIRES that the GROUP NAME be supplied
//It will attempt to add a group record to the database with the given Group Name
//If successfull add's group ID be a positive integer representing it's group id in
//the system, otherwise it will be 0.
//
//This can throw an integer on database connection error, or on duplicate group name
void Database::addGroup(group* add)
{
    QSqlQuery query(*dbs);

    //Group ID field is auto incremented
    query.prepare("insert into groups (GroupName) values (?)");
    query.bindValue(0,*add->getGroupName());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    add->setGroupID(0);
    if (query.numRowsAffected() == 1)
    {
        add->setGroupID(query.lastInsertId().toInt());
    }
}

//This takes a pointer to a group object and REQUIRES that the GROUP NAME OR GROUP ID is supplied
//It will attempt to remove a group that matches either the given group name or group ID.
//It returns a boolean indicating success or faliure.
//
//Please note: if the group is not in the system, this function will return true
//
//It can throw an integer on database connection error
bool Database::removeGroup(group* remove)
{
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare("delete from groups where Gid = ? OR GroupName = ?");
    query.bindValue(0,remove->getGroupID());
    query.bindValue(1,*remove->getGroupName());
    if (query.exec())
    {
        verdict = true;
    }
    else
    {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

//This takes a pointer to a group object and REQUIRES the GROUP NAME be supplied
//It will attempt to discover if a group with the given group name exists.
//If the group is in the system, then the group object is updated with the group ID,
//otherwise the group ID is set to 0.
//
//This can throw and integer on database connection error
void Database::doesGroupExist(group* discover){
    QSqlQuery query(*dbs);
    query.prepare("select Gid from groups where GroupName = ?");
    query.bindValue(0,*discover->getGroupName());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    else
    {
        discover->setGroupID(0);
        if (query.size() == 1)
        {
            query.next();
            discover->setGroupID(query.value(0).toInt());
        }
    }
}

//This takes a pointer to a group object and REQUIRES a GROUP ID
//It also takes a pointer to a person object and REQUIRES a TNUMBER
//It attempts to add the given person to the given group.
//It returns a boolean indicating success. Please note, if the person was already
// a member, this function returns true.
//
//This can throw an int on database connection error, on the group not existing,
//  and on the preson not existing
bool Database::addPersonToGroup(group* g, person* p)
{
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare("insert into p_g (Gid, TNumber) values (?,?)");
    query.bindValue(0,g->getGroupID());
    query.bindValue(1,*p->getTNumber());
    if (query.exec())
    {
        verdict = true;
    }
    else
    {
        int error = getErrorNumber(query.lastError());
        if (error != 4)
        {
            throw error;
        }
        else
        {//error 4 indicates duplicate key, therefore the user was already a part of the group
            verdict = true;
        }
    }
    return verdict;
}

//This takes a pointer to a group object and REQUIRES a GROUP ID be supplied.
//It will attempt to remove all people from the group.
//It returns a boolean on success. Please note, if no people were in the group to
//start with, this will return true.
//
//This can throw an int on database connection error
bool Database::removeAllPeopleFromGroup(group* removeAll){
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare("delete from p_g where Gid = ?");
    query.bindValue(0,removeAll->getGroupID());
    if (query.exec())
    {
        verdict = true;
    }
    else
    {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

//This will attempt to retrieve all groups in the system.
//This returns a pointer to a QList with pointers to group objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any groups in the list are ordered alphabetically by group name.
//All groups returned will have valid group names and group ID's
//
//This can throw an int on database connection error
QList<group*>* Database::getGroups()
{
    QSqlQuery query(*dbs);
    QString queryString = "select GID, GroupName from groups ORDER BY GroupName";

    query.prepare(queryString);

    if(!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    QList<group*> *groups = new QList<group*>;
    while (query.next())
    {
        group* next = new group();
        next->setGroupID(query.value(0).toInt());
        next->setGroupName(query.value(1).toString());
        groups->append(next);
    }
    return groups;
}

//This takes a pointer to a group object and REQUIRES a GROUP ID be supplied
//This will attempt to retrieve all rooms opened by a group in the system.
//This returns a pointer to a QList with pointers to room objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any rooms in the list are ordered alphabetically by room name.
//All rooms returned will have valid room names, and room id's.
//
//This can throw an int on database connection error
QList<room*>* Database::getGroupRooms(group* discover)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select distinct RoomName,RoomID from rooms where ")+
                          QString("RoomID in (select distinct RoomID from g_r where Gid = ?) ORDER BY RoomName");
    query.prepare(queryString);
    query.bindValue(0,discover->getGroupID());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    QList<room*>* rooms = new QList<room*>;
    while (query.next())
    {
        room* get = new room;
        get->setRoomName(query.value(0).toString());
        get->setRoomID(query.value(1).toInt());
        rooms->append(get);
    }
    return rooms;
}

//This takes a pointer to a group object and REQUIRES a GROUP ID be supplied
//This will attempt to retrieve all people who are a member of the given group in the system.
//This returns a pointer to a QList with pointers to person objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any people in the list are ordered alphabetically by last name, first name.
//All people returned will have valid TNumbers, First Names, and Last Names
//
//This can throw an int on database connection error
QList<person*>* Database::getGroupPeople(group* discover)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select distinct TNumber,LastName,FirstName from people where ")+
                          QString("TNumber in (select distinct TNumber from p_g where Gid = ?) ORDER BY LastName,FirstName");
    query.prepare(queryString);
    query.bindValue(0,discover->getGroupID());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    QList<person*>* people = new QList<person*>;
    while (query.next())
    {
        person* get = new person;
        get->setTNumber(query.value(0).toString());
        get->setLastName(query.value(1).toString());
        get->setFirstName(query.value(2).toString());
        people->append(get);
    }
    return people;
}

//////////////////////////////////////////////////////////////////////////
// End Groups
//////////////////////////////////////////////////////////////////////////

//This function takes a RoomAssociationMode indicating for what things rooms are to be modified (Keys or Groups)
//It also takes another RoomAssociationMode indicating what kind of modification (Adding or Removing Rooms)
//The third parameter is an integer giving either the group id or the tag number
//The fourth is a pointer to a QList of pointers to room object, this REQUIRES THAT FOR EACH ROOM a ROOM ID is supplied
//This attempts to either add or remove rooms to either a key or a group.
//This returns a boolean indicating success
//Please note, if adding rooms that already are associated,
// or removing rooms that are NOT associated, this will return true.
//
//The could throw an integer on database connection error, or on the key, group,
//  or room not existing when adding
//
//NOTE: This function executes multiple sql statements, if a transaction is needed
// the caller must perform it.
bool Database::modifyRoomList(Database::RoomAssociationMode table, Database::RoomAssociationMode function, int primaryKey, QList<room *> *rooms)
{
    QString queryString = "";
    QSqlQuery query(*dbs);
    QString primary;

    if (table == Database::TO_KEY)
    {
        queryString += "k_r ";
        primary = "TagNum";
    }
    else if (table == Database::TO_GROUP)
    {
        queryString += "g_r ";
        primary = "Gid";
    }
    else
    {//return false on bad mode
        return false;
    }

    if (function == Database::ADD_ROOMS)
    {
        //insert information from above
        queryString = "insert into "+queryString+"("+primary+", RoomID) values (?, ?)";
        query.prepare(queryString);
        query.bindValue(0,primaryKey);
        bool err = false;
        for (int i = 0; i < rooms->count(); i++)
        {//we bind a different value for each key and re-exec prepared statement for each room
            room* add = rooms->at(i);
            query.bindValue(1,add->getRoomID());
            if (!query.exec())
            {
                 int error = getErrorNumber(query.lastError());
                 if (error != 4)
                 {//again if the room is already with the thing, ignore
                     throw error;
                 }
                 continue;
            }
            if (query.numRowsAffected() != 1)
            {//If a room was not added.
                err = true;
                break;
            }
        }
        return !err;
    }
    else if (function == Database::REMOVE_ROOMS)
    {
        //insert information from above
        queryString = "delete from "+queryString+"where "+primary+" = ? AND RoomID = ?";
        query.prepare(queryString);
        query.bindValue(0,primaryKey);
        for (int i = 0; i < rooms->count(); i++)
        {//we bind a different value for each key and re-exec prepared statement for each room
            room* add = rooms->at(i);
            query.bindValue(1,add->getRoomID());
            if (!query.exec())
            {
                 throw getErrorNumber(query.lastError());
            }

        }
        return true;
    }
    else
    {//return false on bad mode
        return false;
    }
}
