#include "database.h"
#include "person.h"
#include "key.h"
#include "category_item.h"

// Note: this file is very large, and that is because almost all of these functions
// deal with inventory items and as such can handle both keys and other inventory.
// MORAL THROUGHOUT THIS FILE: Keys are special

//This function parses through the list of tables in the database
//and returns a possibly empty QStringList with the names of other inventory categories
QStringList Database::getCategory()
{

    qDebug() << "Getting tables";
    QStringList list = dbs->tables();
    QStringList ret;
    qDebug() << "Looping tables";
    foreach (QString str,list){
        //Each category has a telltale p_categoryName table
        if (str.contains("p_",Qt::CaseInsensitive)){
            //but ignore p_g and p_k
            if (str != "p_g" && str != "p_k"){
                ret.append(str.remove(0,2));
            }
        }
    }

    return ret;
}

//This function takes a qstring indicating the wanted category name
//This attempts to add the tables and triggers necessary to use the new category
//This returns a boolean indicating success
//
//The could throw an integer on database connection error, or on duplicate table name
//  or one the use of a restricted table name
//
//NOTE: This function executes multiple sql statements, if a transaction is needed
// the caller must perform it.
bool Database::addCategory(QString catName)
{
    QSqlQuery query(*dbs);

    QString cmd = "create table "+catName+" (Descriptor varchar(50) primary key, Description varchar(255) DEFAULT 'No description given.', Total int DEFAULT 1, InInv int DEFAULT 1) Engine = Innodb";
    query.prepare(cmd);
    if (!query.exec())
    {
        QSqlError err = query.lastError();
        //If a mysql restricted table name is used a syntax error is generated
        if (err.number() == 1064)
        {
            throw 250;
        }
        throw getErrorNumber(err);
    }
    cmd = "create table IF NOT EXISTS p_"+catName+" (TransactionID int primary key AUTO_INCREMENT, Tnumber char(9) DEFAULT 'T00000000', Descriptor varchar(50) DEFAULT 'None.', NumOut int DEFAULT 1, DateOut date, DateDue date DEFAULT null, FOREIGN KEY (Tnumber) REFERENCES people(Tnumber), FOREIGN KEY (Descriptor) REFERENCES "+catName+" (Descriptor) ON DELETE cascade) Engine=Innodb";
    query.prepare(cmd);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }

    query.exec("set DELIMITER = \\");
    cmd = "create trigger now_date_p_"+catName+" BEFORE INSERT ON p_"+catName+" FOR EACH ROW set NEW.DateOut=now()";
    query.prepare(cmd);
    if (!query.exec())
    {
        qDebug() << "Trigger 1" << endl;
        int err = getErrorNumber(query.lastError());
        if (err != 5)
        {
            throw err;
        }
    }

    cmd = "create trigger checkout_"+catName+" AFTER INSERT ON p_"+catName+" FOR EACH ROW BEGIN DECLARE num INT; set num = NEW.NumOut; update "+catName+" set InInv = InInv - num where Descriptor = new.Descriptor; END";
    query.prepare(cmd);
    if (!query.exec())
    {
        qDebug() << cmd;
        qDebug() << "Trigger 2" << endl;

        int err = getErrorNumber(query.lastError());
        if (err != 5)
        {
            throw err;
        }
    }
    cmd = "create trigger change_"+catName+" AFTER UPDATE ON p_"+catName+" FOR EACH ROW BEGIN DECLARE num INT; set num = OLD.NumOut - NEW.NumOut; update "+catName+" set InInv = InInv + num where Descriptor = new.Descriptor; END";
    query.prepare(cmd);
    if (!query.exec())
    {
        qDebug() << cmd;
        qDebug() << "Trigger 3" << endl;

        int err = getErrorNumber(query.lastError());
        if (err != 5)
        {
            throw err;
        }
    }
    cmd = "create trigger checkin_"+catName+" AFTER DELETE ON p_"+catName+" FOR EACH ROW BEGIN DECLARE num INT; set num = OLD.NumOut; update "+catName+" set InInv = InInv + num where Descriptor = OLD.Descriptor; END";
    query.prepare(cmd);
    if (!query.exec())
    {
        qDebug() << cmd;
        qDebug() << "Trigger 4" << endl;

        int err = getErrorNumber(query.lastError());
        if (err != 5)
        {
            throw err;
        }
    }
    query.exec("set DELIMITER = ;");
    return true;

}

//This function takes a qstring indicating the wanted category name
//This attempts to remove the tables and triggers necessary to use the category
//This returns a boolean indicating success
//
//The could throw an integer on database connection error
//
//NOTE: This function executes multiple sql statements, if a transaction is needed
// the caller must perform it.
bool Database::removeCategory(QString catName)
{
    QSqlQuery query(*dbs);
    query.prepare("drop trigger checkin_"+catName);
    query.bindValue(0,"checkin_"+catName);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    query.prepare("drop trigger change_"+catName);
    query.bindValue(0,"change_"+catName);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    query.prepare("drop trigger checkout_"+catName);
    query.bindValue(0,"checkout_"+catName);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    query.prepare("drop trigger now_date_p_"+catName);
    query.bindValue(0,"checkout_"+catName);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    query.prepare("drop table p_"+catName);
    query.bindValue(0,"p_"+catName);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    query.prepare("drop table "+catName);
    query.bindValue(0,catName);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    return true;
}

//This takes a pointer to a inventory object and REQUIRES the TABLE, TAG NUM/DESCRIPTOR,
//  KEY NAME/DESCRIPTION, TOTAL, AND CURRENT NUMBER be valid, current number
//  indicates the amount available to checkout
//It will attempt to add a record to the database dependant upon the table value
//This returns a boolean indicating success
//
//This can throw an integer on database connection error, or on duplicate
//  descriptor, or tag number-key name, it could also throw an error about
//  the category no longer existing
bool Database::addInventoryItem(inventory* inv)
{
    QString queryString = "insert into ";
    QString table = *inv->getTable();
    QStringList params;
    if (table == "pkeys")
    {//keys are special
        //key
        queryString += "pkeys (TagNum, KeyName, Total, InInv) values (?,?,?,?)";
        key* k = (key*)inv;
        QString tagNum;
        tagNum = tagNum.setNum(k->getTagNum());
        params.append(tagNum);
        params.append(*k->getKeyName());
    }
    else
    {
        //other
        queryString += table+" (Descriptor, Description, Total, InInv) values (?,?,?,?)";
        category_item* ci = (category_item*)inv;
        params.append(*ci->getDescriptor());
        params.append(*ci->getDescription());
    }
    qDebug() << queryString;
    //for integer conversion
    QString num;
    num = num.setNum(inv->getTotal());
    params.append(num);
    num = num.setNum(inv->getCurrentNumber());
    params.append(num);
    QSqlQuery query(*dbs);
    query.prepare(queryString);
    //bind all of our paramaters
    for( int i = 0; i < params.count(); i++)
    {
        query.bindValue(i,params.at(i));
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    if (query.numRowsAffected() == 0)
    {//query execed but no rows added
        return false;
    }
    return true;

}

//This takes a pointer to a inventory object and REQUIRES the TABLE,
//  TAG NUM/DESCRIPTOR, be valid
//It will attempt to remove the record from the database dependant upon the table value
//This returns a boolean indicating success
//
//Please note: if the item was not in the database, this function will return true
//
//This can throw an integer on database connection error, it could also throw
//  an error about the category no longer existing
bool Database::removeInventoryItem(inventory* inv)
{
    QString queryString = "delete from ";
    QString table = *inv->getTable();
    QStringList params;
    if (table == "pkeys")
    {
        //key
        queryString += "pkeys where TagNum = ?";
        key* k = (key*)inv;
        QString num;
        num = num.setNum(k->getTagNum());
        params.append(num);
    }
    else
    {
        //other
        queryString += table+" where Descriptor = ?";
        category_item* ci = (category_item*)inv;
        params.append(*ci->getDescriptor());
    }
    QSqlQuery query(*dbs);
    query.prepare(queryString);
    for( int i = 0; i < params.count(); i++)
    {
        query.bindValue(i,params.at(i));
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    return true;

}

//This takes a pointer to a inventory object and REQUIRES the TABLE,
//  TAG NUM/DESCRIPTOR, NUMBER TO CHECKOUT, AND DUEDATE be valid, although due date
//  could be 0000-00-00 indicating no due date
//It also takes a pointer to a person object and REQUIRES THE TNUMBER be valid
//It will attempt to add a record in the p_TABLE table indicating the given person
//  has checked out the item
//This returns a boolean indicating success, and sets the check-out transaction number
//  in the inventory object to a non 0 integer on success
//
//Please note: a database trigger will update the number available for checkout
//
//This can throw an integer on database connection error, it could also throw
//  an error about the category no longer existing, if the person does not exist,
//  or if the item to check-out does not exist
bool Database::CheckOutInventory(inventory* inv, person* p)
{
    QString queryString = "insert into ";
    QString table = *inv->getTable();
    QStringList params;
    QSqlQuery query(*dbs);
    if (table == "pkeys")
    {
        //key
        key* k = (key*)inv;

        queryString += "p_k (TagNum, ";
        QString num;
        num = num.setNum(k->getTagNum());
        params.append(num);
    }
    else
    {
        //other
        category_item* ci = (category_item*)inv;

        queryString += "p_"+table+" (Descriptor, ";
        params.append(*ci->getDescriptor());

    }
    queryString += "TNumber, NumOut";
    QString toBind = "(?,?,?";
    params.append(*p->getTNumber());
    QString num;
    num.setNum(inv->getCurrentNumber());
    params.append(num);

    //if the date given matches 0000-00-00 do not insert a due date,
    // the database will insert NULL which is what I want
    QDate dueDate = QDate::fromString(inv->getDateDue(),Qt::ISODate);
    QDate nullDate(0,0,0);
    if (dueDate != nullDate)
    {
        queryString += ", DateDue";
        toBind += ",?";
        params.append(inv->getDateDue());
    }


    queryString += ") values "+toBind+" )";
qDebug() << queryString;
    query.prepare(queryString);
    for( int i = 0; i < params.count(); i++)
    {
        query.bindValue(i,params.at(i));
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    inv->setCheckOutTransaction(0);
    if (query.numRowsAffected() != 1)
    {
        return false;
    }
    inv->setCheckOutTransaction(query.lastInsertId().toInt());
    return true;

}


//This takes a pointer to a inventory object and REQUIRES the TABLE,
//  TRANSACTION ID, NUMBER CHECKED OUT, AND CURRENT NUMBER be valid, the current
//  number here indicates the number of that inventory to check-in
//If the current number is less than the number checked out, then the record is
//  updated to show that some of those items have been checked out. Else if
//  the current number is equal to the number checked-out, the record is removed,
//  indicating that all of the item was checked in.
//This returns a boolean indicating success
//
//Please note: a database trigger will update the number available for checkout
//Please note: if the check-out does not exist and all of that item is being
//  checked-in, this will return true.
//
//This can throw an integer on database connection error, it could also throw
//  an error about the category no longer existing, or if the item to check-in does not exist
bool Database::CheckInInventory(inventory* inv)
{
    QString queryString;
    QSqlQuery query(*dbs);
    QStringList params;
    QString num;
    if (inv->getCurrentNumber() != inv->getNumberCheckedOut())
    {   //update record
        queryString = " set NumOut = NumOut - ? where TransactionID = ?";
        if (*inv->getTable() == "pkeys")
        {
            //keys
            queryString = "update p_k"+queryString;
            num = num.setNum(inv->getCurrentNumber());
            params.append(num);
            num = num.setNum(inv->getCheckOutTransaction());
            params.append(num);
        }
        else
        {
            //other
            queryString = "update p_"+*inv->getTable()+queryString;
            num = num.setNum(inv->getCurrentNumber());
            params.append(num);
            num = num.setNum(inv->getCheckOutTransaction());
            params.append(num);

        }
        query.prepare(queryString);
        for (int i = 0; i < params.count(); i++)
        {
            query.bindValue(i,params.at(i));
        }
        qDebug() << queryString;
        if (!query.exec())
        {
            throw getErrorNumber(query.lastError());
        }
        if (query.numRowsAffected() == 0)
        {
            return false;
        }
        return true;
    }
    else
    {   //delete record
        queryString = " where TransactionID = ?";
        if (*inv->getTable() == "pkeys")
        {
            //keys
            queryString = "delete from p_k"+queryString;
            num = num.setNum(inv->getCheckOutTransaction());
            params.append(num);
        }
        else
        {
            //other
            queryString = "delete from p_"+*inv->getTable()+queryString;
            num = num.setNum(inv->getCheckOutTransaction());
            params.append(num);

        }
        query.prepare(queryString);
        for (int i = 0; i < params.count(); i++)
        {
            query.bindValue(i,params.at(i));
        }
        if (!query.exec())
        {
            throw getErrorNumber(query.lastError());
        }
        return true;
    }

}

//This takes a pointer to a inventory object and REQUIRES the TABLE,
//  TAG NUM/DESCRIPTOR, and CURRENT NUMBER be valid, current number is the number
//  of items to add or remove from the database
//It will attempt to update the total and amount available subject to the following:
//  The amount available for check-out cannot be negative,
//  If an item's total quantitiy reaches 0 it is removed from the system
//This returns a boolean indicating success
//
//
//This can throw an integer on database connection error, it could also throw
//  an error about the category no longer existing, the item not existing, or the,
//  removal causing the amount available to go negative
bool Database::updateItemQuantity(inventory* inv)
{
    QString queryString = "update ";
    QString table = *inv->getTable();
    QStringList params;
    QSqlQuery query(*dbs);
    if (table == "pkeys")
    {
        //key
        key* k = (key*)inv;
        if (k->getCurrentNumber() < 0)
        {//special stuff if removing
            query.prepare("select InInv,Total from pkeys where TagNum = ?");
            QString tagNum;
            tagNum = tagNum.setNum(k->getTagNum());
            query.bindValue(0,tagNum);
            if (!query.exec())
            {
                throw getErrorNumber(query.lastError());
            }
            if (query.size() == 0)
            {
                //not found
                throw 200;
            }
            query.next();
            if (query.value(0).toInt() + k->getCurrentNumber() < 0){
                //cant remove checked out items
                throw 300;
            }
            if (query.value(1).toInt() + k->getCurrentNumber() == 0){
                //would reduce the inventory to 0
                //delete item
                inv->setAmountAvalable(0);
                return removeInventoryItem(inv);
            }
        }

        queryString += "pkeys set Total = Total + ?, InInv = InInv + ? where TagNum = ?";
        QString num;
        num = num.setNum(k->getCurrentNumber());
        params.append(num);
        params.append(num);
        num = num.setNum(k->getTagNum());
        params.append(num);
    }
    else
    {
        //other
        category_item* ci = (category_item*)inv;
        if (ci->getCurrentNumber() < 0)
        {//again special things if removing items
            query.prepare("select InInv,Total from "+*inv->getTable()+" where Descriptor = ?");
            query.bindValue(0,*ci->getDescriptor());
            if (!query.exec())
            {
                throw getErrorNumber(query.lastError());
            }
            if (query.size() == 0)
            {
                //not found
                throw 200;
            }
            query.next();
            if (query.value(0).toInt() + ci->getCurrentNumber() < 0)
            {
                //cant remove checked out items
                throw 300;
            }
            if (query.value(1).toInt() + ci->getCurrentNumber() == 0)
            {
                //would reduce the inventory to 0
                inv->setAmountAvalable(0);
                return removeInventoryItem(inv);
            }
        }

        queryString += *inv->getTable()+" set Total = Total + ?, InInv = InInv + ? where Descriptor = ?";
        QString num;
        num = num.setNum(ci->getCurrentNumber());
        params.append(num);
        params.append(num);
        params.append(*ci->getDescriptor());

    }

    query.prepare(queryString);
    for( int i = 0; i < params.count(); i++)
    {
        query.bindValue(i,params.at(i));
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    if (query.numRowsAffected() == 0)
    {
        //not found
        throw 200;
    }
    //This tells the calling function what was changed, because so much can happen
    inv->setAmountAvalable(inv->getAmountAvailable()+inv->getCurrentNumber());
    return true;

}

//This takes a pointer to an inventory object and REQUIRES the TAG NUMBER/DESCRIPTOR
//  and DATE CHECKED OUT be supplied
//It also takes a pointer to a person object and REQUIRES the TNUMBER be supplied
//It will attempt to discover if the item with the specified primary key and
//  date checked out has been checked out by the person given.
//If the check-out is in the system, then the inventory object is updated with
//  the correct Transaction ID,
//otherwise the transaction ID is set to 0.
//
//This can throw an integer on database connection error, it could also throw
//  an error about the category no longer existing
void Database::doesCheckOutExist(inventory* item, person* p)
{
    QStringList params;
    QSqlQuery query(*dbs);
    QString queryString = "select distinct TransactionID";

    QString table = *item->getTable();
    if (table == "pkeys")
    {

        //key
        queryString += " from p_k where TagNum = ? ";
        key* k = (key*)item;
        QString num;
        num = num.setNum(k->getTagNum());
        params.append(num);
    }
    else
    {
        //other
        queryString += " from p_"+*item->getTable()+" where Descriptor = ? ";
        category_item* ci = (category_item*)item;
        params.append(*ci->getDescriptor());
    }
    queryString += "AND TNumber = ? AND DateOut = ?";
    params.append(*p->getTNumber());
    params.append(item->getDateOut());
    query.prepare(queryString);

    for (int i = 0; i < params.count(); i++)
    {
        query.bindValue(i,params.at(i));
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    if (query.size() != 1)
    {
        item->setCheckOutTransaction(0);
        return;
    }
    query.next();
    item->setCheckOutTransaction(query.value(0).toInt());
}

//This takes a pointer to an inventory object and REQUIRES the TRANSACTION ID
//It will attempt to discover the information about a check-out in the sysetm
//If the check-out is in the system, then the inventory object is updated with
//  the correct Tag Number/Descriptor, Number checked out, check out date, and due date
//
//This can throw an integer on database connection error, it could also throw
//  an error about the category no longer existing, or the check-out not existing
void Database::getCheckOutInfo(inventory *item, person *p)
{
    QStringList params;
    QSqlQuery query(*dbs);
    QString queryString = "select distinct TransactionID,NumOut,DateOut,DateDue";

    QString table = *item->getTable();
    if (table == "pkeys")
    {
        //key
        queryString += ",TagNum from p_k where TransactionID = ? ";
    }
    else
    {
        //other
        queryString += ",Descriptor from p_"+*item->getTable()+" where TransactionID = ? ";
    }

    query.prepare(queryString);
    query.bindValue(0,item->getCheckOutTransaction());

    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    if (query.size() != 1)
    {
        //not found;
        throw 200;
    }
    query.next();
    if (table == "pkeys")
    {
        ((key*)item)->setCheckOutTransaction(query.value(0).toInt());
        ((key*)item)->setNumberCheckedOut(query.value(1).toInt());
        ((key*)item)->setDateOut(query.value(2).toString());
        ((key*)item)->setDateDue(query.value(3).toString());
        ((key*)item)->setTagNum(query.value(4).toInt());

    }
    else
    {
        ((category_item*)item)->setCheckOutTransaction(query.value(0).toInt());
        ((category_item*)item)->setNumberCheckedOut(query.value(1).toInt());
        ((category_item*)item)->setDateOut(query.value(2).toString());
        ((category_item*)item)->setDateDue(query.value(3).toString());
        ((category_item*)item)->setDescriptor(query.value(4).toString());

    }
}

//This takes a pointer to an inventory object and REQUIRES the TAG NUMBER/DESCRIPTOR
//It will attempt to discover the information about an inventory item in the sysetm
//If the item is in the system, then the inventory object is updated with
//  the correct Tag Number/Descriptor, total inventory, and amount available for
//  check-out
//
//This can throw an integer on database connection error, it could also throw
//  an error about the category no longer existing, or the item not existing
void Database::getInventoryInfo(inventory* item)
{
    QSqlQuery query(*dbs);
    QString queryString;
    QString param;
    bool keyItem;
    if (*item->getTable() == "pkeys")
    {
        keyItem = true;
        queryString = "select distinct TagNum,Total,InInv,KeyName from pkeys where TagNum = ?";
        param = param.setNum(((key*)item)->getTagNum());
    }
    else {
        keyItem = false;
        queryString = "select distinct Description,Total,InInv from "+*item->getTable()+
                      " where Descriptor = ?";
        param = *((category_item*)item)->getDescriptor();
    }

    qDebug() << queryString;
    query.prepare(queryString);

    query.bindValue(0,param);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    if (query.size() == 0)
    {
        //not found
        throw 200;
    }
    query.next();

    if (keyItem)
    {
        ((key*)item)->setTagNum(query.value(0).toInt());
        ((key*)item)->setTotal(query.value(1).toInt());
        ((key*)item)->setAmountAvalable(query.value(2).toInt());
        ((key*)item)->setKeyName(query.value(3).toString());

    }
    else
    {
        ((category_item*)item)->setDescription(query.value(0).toString());
        ((category_item*)item)->setTotal(query.value(1).toInt());
        ((category_item*)item)->setAmountAvalable(query.value(2).toInt());

    }

}

//This takes a pointer to a category_item object and REQUIRES the TABLE
//This function attempts to find all category items in the system that matches the given
//  descriptor and/or description with the mysql LIKE '%descriptrion%' or
//  LIKE '%description%'
//If only the table is given, this returns all items in that category
//The function returns a pointer to a QList with pointers to category_item objects
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any category_items in the list are ordered alphabetically
//  by descriptor.
//All items returned will have valid descriptors, descriptions, totals,
//  available amounts, and table names
//
//This can throw an int on database connection error, or if the category no
//  longer exists.
QList<category_item*>* Database::getItems(category_item* item)
{
    QList<category_item*>* items = new QList<category_item*>;
    QStringList params;
    QSqlQuery query(*dbs);
    QString queryString = "select distinct Descriptor,Description,Total,InInv from "+*item->getTable();

    bool first = true;
    if (*item->getDescriptor() != "")
    {
        if (!first)
        {
            queryString += " AND";
        }
        else
        {
            queryString += " where ";
            first = !first;
        }
        queryString += " Descriptor LIKE ?";
        params.append("%"+*item->getDescriptor()+"%");
    }
    if (*item->getDescription() != "")
    {
        if (!first)
        {
            queryString += " AND";
        }
        else
        {
            queryString += " where ";
            first = !first;
        }
        queryString += " Description LIKE ?";
        params.append("%"+*item->getDescription()+"%");

    }
    queryString += " ORDER BY Descriptor";
    qDebug() << queryString;
    query.prepare(queryString);

    for (int i = 0; i < params.count(); i++)
    {
        query.bindValue(i,params.at(i));
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    while (query.next())
    {
        category_item* ci = new category_item;
        ci->setDescriptor(query.value(0).toString());
        ci->setDescription(query.value(1).toString());
        ci->setTotal(query.value(2).toInt());
        ci->setAmountAvalable(query.value(3).toInt());
        ci->setTable(*item->getTable());
        items->append(ci);
    }
    return items;
}

//This takes a pointer to an inventory object and REQUIRES the TABLE and TAG NUMBER/DESCRIPTOR
//This function attempts to find all people in the system who have the specified
//  key/category_item checked-out.
//The function returns a pointer to a QList with pointers to person objects
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any people in the list are ordered alphabetically
//  by Last Name, First Name
//All people returned will have valid TNumbers, First Names, and Last Names
//
//This can throw an int on database connection error, or if the category no
//  longer exists.
QList<person*>* Database::getPeopleWithInventoryCheckedOut(inventory* inv)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select distinct TNumber,LastName,FirstName from people where ")+
                          QString("TNumber in (select distinct TNumber from");
    if (*inv->getTable() == "pkeys")
    {//keys
        queryString += " p_k where TagNum = ?) ORDER BY LastName,FirstName";
        query.prepare(queryString);
        query.bindValue(0,((key*)inv)->getTagNum());
    }
    else
    {// other
        queryString += " p_"+*inv->getTable()+" where Descriptor = ?) ORDER BY LastName,FirstName";
        query.prepare(queryString);
        query.bindValue(0,*((category_item*)inv)->getDescriptor());
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    QList<person*>* people = new QList<person*>;
    while (query.next())
    {
        person* get = new person;
        get->setTNumber(query.value(0).toString());
        get->setLastName(query.value(1).toString());
        get->setFirstName(query.value(2).toString());
        people->append(get);
    }
    return people;
}
