#include "database.h"
#include "person.h"
#include "group.h"
#include "key.h"
#include "category_item.h"



/////////////////////////////////////////////////////////////////////////////
// People Management
/////////////////////////////////////////////////////////////////////////////

//This takes a pointer to a person object and REQUIRES TNUMBER, FIRST NAME, LAST NAME
//It attempts to add a person to the database with the given Tnumber, First name,
//  Last name, and any other information given in the person object.
//It returns a boolean indicating success.
//
//This can throw an integer on database connection error, or on duplicate TNumber name
bool Database::addPerson(person* to_add)
{
    bool verdict = false;
    QSqlQuery query(*dbs);
    QString queryString = "insert into people (TNumber, FirstName, LastName";
    QString valuesString = "VALUES (?, ?, ?";
    QStringList valuesToBind;

    valuesToBind.append(*to_add->getTNumber());
    valuesToBind.append(*to_add->getFirstName());
    valuesToBind.append(*to_add->getLastName());

    //in these statements we must keep seperate query and values strings because
    //we don't know what is coming up ahead
    if (*to_add->getStreetAddress() != "")
    {
        queryString += ", strAddr";
        valuesString += ", ?";
        valuesToBind.append(*to_add->getStreetAddress());
    }
    if (*to_add->getHomePhone() != "")
    {
        queryString += ", HPhone";
        valuesString += ", ?";
        valuesToBind.append(*to_add->getHomePhone());
    }
    if (*to_add->getWorkPhone() != "")
    {
        queryString += ", WPhone";
        valuesString += ", ?";
        valuesToBind.append(*to_add->getWorkPhone());
    }
    if (*to_add->getCellPhone() != "")
    {
        queryString +=", CPhone";
        valuesString += ", ?";
        valuesToBind.append(*to_add->getCellPhone());
    }
    if (*to_add->getEmail() != "")
    {
        queryString += ", email";
        valuesString += ", ?";
        valuesToBind.append(*to_add->getEmail());
    }
    queryString += ") ";
    valuesString += ")";
    queryString += valuesString;

    query.prepare(queryString);
    for (int i = 0; i < valuesToBind.count(); i++)
    {
        query.bindValue(i,valuesToBind.at(i));
    }
    if( query.exec())
    {
        if (query.numRowsAffected() == 1)
        {
            verdict = true;
        }
    }
    else
    {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

//This takes a pointer to a person object and REQUIRES TNUMBER and AT LEAST ONE OTHER
//It attempts to update a person's information in the database with the optionally
//  given First name, Last name, and any other information given in the person object.
//It returns a boolean indicating success.
//
//This can throw an integer on database connection error, or on no person found
bool Database::updatePerson(person* to_update)
{
    bool verdict = false;
    QSqlQuery query(*dbs);
    QString queryString = "update people set ";
    QStringList valuesToBind;

    bool first = true;

    if (*to_update->getFirstName() != "")
    {
        if (first)
        {
            first = false;
        }
        else
        {
            queryString += ", ";
        }
        queryString += "FirstName = ?";
        valuesToBind.append(*to_update->getFirstName());
    }

    if (*to_update->getLastName() != "")
    {
        if (first)
        {
            first = false;
        }
        else
        {
            queryString += ", ";
        }
        queryString += "LastName = ?";
        valuesToBind.append(*to_update->getLastName());
    }

    if (*to_update->getStreetAddress() != "")
    {
        if (first)
        {
            first = false;
        }
        else
        {
            queryString += ", ";
        }
        queryString += "strAddr = ?";
        valuesToBind.append(*to_update->getStreetAddress());
    }
    if (*to_update->getHomePhone() != "")
    {
        if (first)
        {
            first = false;
        }
        else
        {
            queryString += ", ";
        }
        queryString += "HPhone = ?";
        valuesToBind.append(*to_update->getHomePhone());
    }
    if (*to_update->getWorkPhone() != "")
    {
        if (first)
        {
            first = false;
        }
        else
        {
            queryString += ", ";
        }
        queryString += "WPhone = ?";
        valuesToBind.append(*to_update->getWorkPhone());
    }
    if (*to_update->getCellPhone() != "")
    {
        if (first)
        {
            first = false;
        }
        else
        {
            queryString += ", ";
        }
        queryString += "CPhone = ?";
        valuesToBind.append(*to_update->getCellPhone());
    }
    if (*to_update->getEmail() != "")
    {
        if (first)
        {
            first = false;
        }
        else
        {
            queryString += ", ";
        }
        queryString += "email = ?";
        valuesToBind.append(*to_update->getEmail());
    }
    queryString += " where Tnumber = ?";
    valuesToBind.append(*to_update->getTNumber());


    query.prepare(queryString);
    for (int i = 0; i < valuesToBind.count(); i++)
    {
        query.bindValue(i,valuesToBind.at(i));
    }
    if( query.exec())
    {
        if (query.numRowsAffected() == 1)
        {
            verdict = true;
        }
        else if (query.numRowsAffected() == 0)
        {
            //not found
            throw 200;
        }
    }
    else
    {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

//This takes a pointer to a person object and REQUIRES that the TNUMBER is supplied
//It will attempt to remove a person that matches the given TNumber.
//It returns a boolean indicating success or faliure.
//
//Please note: if the person is not in the system, this function will return true
//
//It can throw an integer on database connection error
bool Database::removePerson(person* to_remove)
{
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare("delete from people where Tnumber = ?");
    query.bindValue(0,*to_remove->getTNumber());
    if (query.exec())
    {
        verdict = true;
    }
    else
    {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

//This takes a PersonQueryMode to indicate how to search for people
//It also takes a pointer to a person object and REQUIRES at LEAST ONE FIELD be
//  valid for the custom mode
//It also takes a pointer to a QStringList with category names for the
//  with_other_inventory mode
// This function attempts to find all people in the system according to the mode
//  given to it, if the mode is custom, then all availble fields in the person
//  object are searched using the mysql LIKE %field% syntax
//The function returns a pointer to a QList with pointers to person objects
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any people in the list are ordered alphabetically
//  by Last Name, First Name.
//All people returned will have valid tag TNumbers, First Names, and Last Names
//
//This can throw an int on database connection error, or if one of the categories
//  does not exist
QList<person*>* Database::getPeople(PersonQueryMode mode, person* to_search, QStringList* categories)
{
    QList<person*> *people = new QList<person*>;
    QStringList params;
    QSqlQuery query(*dbs);
    QString queryString = "select distinct LastName,FirstName,TNumber from people";
    //ALL mode is covered by the last concatonation
    if (mode == CUSTOM)
    {//long bit to discover which fields are given
        queryString += " where";
        bool first = true;
        if (*to_search->getTNumber() != "")
        {
            if (!first)
            {//technically could never happen
                queryString += " AND";
            }
            else
            {
                first = !first;
            }
            queryString += " Tnumber LIKE ?";
            params.append("%"+*to_search->getTNumber()+"%");
        }
        if (*to_search->getFirstName() != "")
        {
            if (!first)
            {
                queryString += " AND";
            }
            else
            {
                first = !first;
            }
            queryString += " FirstName LIKE ?";
            params.append("%"+*to_search->getFirstName()+"%");
        }
        if (*to_search->getLastName() != "")
        {
            if (!first)
            {
                queryString += " AND";
            }
            else
            {
                first = !first;
            }
            queryString += " LastName LIKE ?";
            params.append("%"+*to_search->getLastName()+"%");
        }
        if (*to_search->getStreetAddress() != "")
        {
            if (!first)
            {
                queryString += " AND";
            }
            else
            {
                first = !first;
            }
            queryString += " strAddr LIKE ?";
            params.append("%"+*to_search->getStreetAddress()+"%");
        }
        if (*to_search->getHomePhone() != "")
        {
            if (!first)
            {
                queryString += " AND";
            }
            else
            {
                first = !first;
            }
            queryString += " HPhone LIKE ?";
            params.append("%"+*to_search->getHomePhone()+"%");
        }
        if (*to_search->getWorkPhone() != "")
        {
            if (!first)
            {
                queryString += " AND";
            }
            else
            {
                first = !first;
            }
            queryString += " WPhone LIKE ?";
            params.append("%"+*to_search->getWorkPhone()+"%");
        }
        if (*to_search->getCellPhone() != "")
        {
            if (!first)
            {
                queryString += "AND";
            }
            else
            {
                first = !first;
            }
            queryString += " CPhone like ?";
            params.append("%"+*to_search->getCellPhone()+"%");
        }
        if (*to_search->getEmail() != "")
        {
            if (!first)
            {
                queryString += " AND";
            }
            else
            {
                first = !first;
            }
            queryString += " email LIKE ?";
            params.append("%"+*to_search->getEmail()+"%");
        }
    }
    if (mode == WITH_KEYS)
    {
        queryString += " where Tnumber in (select distinct Tnumber from p_k)";
    }
    if (mode == IN_GROUPS)
    {
        queryString += " where Tnumber in (select distinct Tnumber from p_g)";
    }
    if (mode == WITH_OTHER_ITEMS)
    {
        //forms a number of subqueries to find people with catgory items checked out
        if (categories->count() == 0)
        {
            //needs to be empty
            return people;
        }
        QString categoryString = " where ";
        bool first = true;
        for (int i = 0; i < categories->count(); i++)
        {
            if (first)
            {
                first = false;
            }
            else
            {
                categoryString += " OR ";
            }
            categoryString += "Tnumber in (select distinct Tnumber from p_"+categories->at(i)+")";

        }
        queryString += categoryString;
    }
    //to find all people, use no where clause and put the ORDER BY segment on the end
    queryString += " ORDER BY LastName, FirstName";

qDebug() << queryString;
    query.prepare(queryString);

    for (int i = 0; i < params.count(); i++)
    {
        query.bindValue(i,params.at(i));
    }
    if(!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    while (query.next())
    {
        person* next = new person();
        next->setLastName(query.value(0).toString());
        next->setFirstName(query.value(1).toString());
        next->setTNumber(query.value(2).toString());
        people->append(next);
    }
    return people;
}

//This takes a pointer to an person object and REQUIRES the TNUMBER
//It will attempt to discover the information about a person in the sysetm
//If the item is in the system, then the inventory object is updated with
//  the correct first name,last name,TNumber, street Address, home phone number,
//  work phone number, cell phone number, and e-mail address
//
//This can throw an integer on database connection error, or about the person not
//  existing in the system
void Database::getPersonInfo(person* fill)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select FirstName, LastName, TNumber, strAddr, HPhone, WPhone, CPhone, email from people")
                          + QString(" where TNumber = ?");
    query.prepare(queryString);
    query.bindValue(0,*fill->getTNumber());

    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    if (query.size() != 1)
    {
        //zero found, no t-number
        throw 200;
    }
    query.next();
    fill->setFirstName(query.value(0).toString());
    fill->setLastName(query.value(1).toString());
    fill->setTNumber(query.value(2).toString());
    fill->setStreetAddress(query.value(3).toString());
    fill->setHomePhone(query.value(4).toString());
    fill->setWorkPhone(query.value(5).toString());
    fill->setCellPhone(query.value(6).toString());
    fill->setEmail(query.value(7).toString());


}

//This takes a pointer to a person object and REQUIRES a TNUMBER be supplied
//This will attempt to retrieve all groups the given person is a member of.
//This returns a pointer to a QList with pointers to group objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any groups in the list are ordered alphabetically by group name.
//All groups returned will have valid group names, and group id's.
//
//This can throw an int on database connection error
QList<group*>* Database::getPersonGroups(person* p)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select distinct GroupName,Gid from groups where ")+
                          QString("Gid in (select distinct Gid from p_g where Tnumber = ?)");
    query.prepare(queryString);
    query.bindValue(0,*p->getTNumber());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    QList<group*>* groups = new QList<group*>;
    while (query.next())
    {
        group* grp = new group;
        grp->setGroupName(query.value(0).toString());
        grp->setGroupID(query.value(1).toInt());
        groups->append(grp);
    }
    return groups;
}

//This takes a pointer to a person object and REQUIRES a TNUMBER be supplied
//It also takes a QStringList with the categories in the system
//This will attempt to retrieve all inventory the given person has checked-out.
//This returns a pointer to a QList with pointers to inventory objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any inventory in the list are ordered alphabetically
//  by KeyName,Descriptor.
//All inventory returned will have valid Tag Numbers/Descriptors, Key Names/Desctiptions,
//  Number checked out, Transaction ID's, date checked out, and (possibly 0000-00-00),
//  due dates
//
//This can throw an int on database connection error, or if a category does not exist
//
//NOTE: This function executes multiple sql statements, if a transaction is needed
// the caller must perform it.
QList<inventory*>* Database::getPersonInv(person* p, QStringList categories)
{
    QList<inventory*>* inv = new QList<inventory*>;
    QSqlQuery query(*dbs);
    //keys
    //The keys section is a bit of a doozy, because the Key name is not the primary key
    //This first query is to get all of the tag numbers and key names, please remember this
    QString queryString = QString("select distinct TagNum,KeyName from pkeys")+
                          QString(" where TagNum in (select distinct TagNum from p_k where Tnumber = ?)");
    query.prepare(queryString);
    query.bindValue(0,*p->getTNumber());
    if (!query.exec())
    {
        delete inv;
        throw getErrorNumber(query.lastError());
    }
    if (query.size() != 0)
    {
        //This query is to get all of the other stuff
        QSqlQuery query2(*dbs);
        query2.prepare("select NumOut,TransactionID,DateOut,DateDue from p_k where TagNum = ? AND Tnumber = ?");
        //While ther are still original query stuff to go through
        while (query.next())
        {

            //bind stuff, including the Tag number from the first query
            query2.bindValue(0,query.value(0).toInt());
            query2.bindValue(1,*p->getTNumber());
            if (!query2.exec())
            {
                qDeleteAll(inv->begin(),inv->end());
                delete inv;
                throw getErrorNumber(query.lastError());
            }

            if (query2.size() < 1)
            {//should never happen because we just pulled from p_k
                qDeleteAll(inv->begin(),inv->end());
                delete inv;
                //not found
                throw 200;
            }

            while (query2.next())
            {//this is a loop because a person can have the same type
                //of key checked out on different days(theoretically)
                key* add = new key;
                add->setTable("pkeys");
                add->setTagNum(query.value(0).toInt());
                add->setKeyName(query.value(1).toString());

                add->setNumberCheckedOut(query2.value(0).toInt());
                add->setCheckOutTransaction(query2.value(1).toInt());
                add->setDateOut(query2.value(2).toString());
                if (!query2.value(3).isNull())
                {
                    add->setDateDue(query2.value(3).toString());
                }
                else
                {//set sentinal value of 0000-00-00 if null
                    add->setDateDue("0000-00-00");
                }
                inv->append(add);
            }
        }
    }
    query.clear();
    //other inventory is more clear because the descriptor is the primary key
    for (int i = 0; i < categories.count(); i++)
    {
        queryString = QString("select distinct descriptor,NumOut,TransactionID,DateOut,DateDue from p_"+categories.at(i)+" where TNumber = ?");
        query.prepare(queryString);
        query.bindValue(0,*p->getTNumber());
        if (!query.exec())
        {
            qDeleteAll(inv->begin(),inv->end());
            delete inv;
            throw getErrorNumber(query.lastError());
        }
        while (query.next())
        {//loop for the same reason as with keys
            category_item* add = new category_item;
            add->setTable(categories.at(i));
            add->setDescriptor(query.value(0).toString());
            add->setNumberCheckedOut(query.value(1).toInt());
            add->setCheckOutTransaction(query.value(2).toInt());
            add->setDateOut(query.value(3).toString());
            if (!query.value(4).isNull())
            {
                add->setDateDue(query.value(4).toString());
            }
            else
            {
                add->setDateDue("0000-00-00");
            }
            inv->append(add);
        }

    }
    return inv;
}

//This takes a pointer to a person object and REQUIRES that the TNUMBER is supplied
//It also takes a pointer to a group object and REQUIRES the GROUP ID.
//It will attempt to remove the selected person from the selected group
//It returns a boolean indicating success or faliure.
//
//Please note: if the person is not in the system, or the group is not in the system,
//  this function will return true
//
//It can throw an integer on database connection error
bool Database::removePersonFromGroup(group* g, person* p)
{
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare("delete from p_g where Tnumber = ? AND Gid = ?");
    query.bindValue(0,*p->getTNumber());
    query.bindValue(1,g->getGroupID());
    if (query.exec())
    {
        verdict = true;
    }
    else
    {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

////////////////////////////////////////////////////////////////////////////
// End People Management
///////////////////////////////////////////////////////////////////////////

