#include "database.h"
#include "room.h"
#include "group.h"
#include "key.h"
#include "person.h"

///////////////////////////////////////////////////////////////////////////
// Rooms
///////////////////////////////////////////////////////////////////////////

//This takes a pointer to a room object and REQUIRES that the ROOM NAME be supplied
//It will attempt to add a room record to the database with the given Room Name
//If successfull add's room ID be a positive integer representing it's room id in
//the system, otherwise it will be 0.
//
//This can throw an integer on database connection error, or on duplicate room name
void Database::addRoom(room* add)
{
    QSqlQuery query(*dbs);

    query.prepare("insert into rooms (RoomName) values (?)");
    query.bindValue(0,*add->getRoomName());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    add->setRoomID(0);
    if (query.numRowsAffected() == 1)
    {
        add->setRoomID(query.lastInsertId().toInt());
    }
}

//This takes a pointer to a room object and REQUIRES that the ROOM NAME OR ROOM ID is supplied
//It will attempt to remove a room that matches either the given room name or room ID.
//It returns a boolean indicating success or faliure.
//
//Please note: if the room is not in the system, this function will return true
//
//It can throw an integer on database connection error, and on the room not existing
bool Database::removeRoom(room* remove)
{
    bool verdict = false;
    QSqlQuery query(*dbs);
    query.prepare("delete from rooms where RoomName = ? or RoomID = ?");
    query.bindValue(0,*remove->getRoomName());
    query.bindValue(1,remove->getRoomID());
    if (query.exec())
    {
        verdict = true;
        if (query.numRowsAffected() == 0)
        {
            //not found
            throw 200;
        }
    }
    else
    {
        throw getErrorNumber(query.lastError());
    }
    return verdict;
}

//This takes a pointer to a room object and REQUIRES the ROOM NAME be supplied
//It will attempt to discover if a room with the given room name exists.
//If the room is in the system, then the room object is updated with the room ID,
//otherwise the room ID is set to 0.
//
//This can throw and integer on database connection error
void Database::doesRoomExist(room* discover)
{
    QSqlQuery query(*dbs);
    query.prepare("select RoomID from rooms where RoomName = ?");
    query.bindValue(0,*discover->getRoomName());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    else
    {
        discover->setRoomID(0);
        if (query.size() == 1)
        {
            query.next();
            discover->setRoomID(query.value(0).toInt());
        }
    }
}

//This will attempt to retrieve all rooms in the system.
//This returns a pointer to a QList with pointers to room objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any rooms in the list are ordered alphabetically by room name.
//All rooms returned will have valid room names and room ID's
//
//This can throw an int on database connection error
QList<room*>* Database::getRooms(){
    QList<room*> *rooms = new QList<room*>;
    QSqlQuery query(*dbs);
    QString queryString = "select RoomID, RoomName from rooms ORDER BY RoomName";

    query.prepare(queryString);

    if(!query.exec()){
        throw getErrorNumber(query.lastError());
    }
    while (query.next()){
        room* next = new room();
        next->setRoomID(query.value(0).toInt());
        next->setRoomName(query.value(1).toString());
        rooms->append(next);
    }
    return rooms;
}

//This takes a pointer to a room object and REQUIRES a ROOM ID be supplied
//This will attempt to retrieve all groups that open this room in the system.
//This returns a pointer to a QList with pointers to group objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any groups in the list are ordered alphabetically by group name.
//All groups returned will have valid group names, and group id's.
//
//This can throw an int on database connection error
QList<group*>* Database::getRoomGroups(room* discover)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select distinct GroupName,Gid from groups where ")+
                          QString("Gid in (select distinct Gid from g_r where RoomID = ?)");
    query.prepare(queryString);
    query.bindValue(0,discover->getRoomID());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    QList<group*>* groups = new QList<group*>;
    while (query.next())
    {
        group* grp = new group;
        grp->setGroupName(query.value(0).toString());
        grp->setGroupID(query.value(1).toInt());
        groups->append(grp);
    }
    return groups;
}

//This takes a pointer to a room object and REQUIRES a ROOM ID be supplied
//This will attempt to retrieve all keys that open this room in the system.
//This returns a pointer to a QList with pointers to key objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any keys in the list are ordered alphabetically by key name.
//All groups returned will have valid key names, and tag numbers.
//
//This can throw an int on database connection error
QList<key*>* Database::getRoomKeys(room* discover)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select distinct KeyName,TagNum from pkeys where ")+
                          QString("TagNum in (select distinct TagNum from k_r where RoomID = ?)");
    query.prepare(queryString);
    query.bindValue(0,discover->getRoomID());
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    QList<key*>* keys = new QList<key*>;
    while (query.next())
    {
        key* k = new key;
        k->setKeyName(query.value(0).toString());
        k->setTagNum(query.value(1).toInt());
        keys->append(k);
    }
    return keys;
}

//This takes a RoomQueryMode indicating which ways a person my access a room
//It also takes a pointer to a room object and REQUIRES a ROOM ID be supplied
//This will attempt to retrieve all people that have access to this room in
//  the system as specified by the mode.
//This returns a pointer to a QList with pointers to person objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any people in the list are ordered alphabetically
//  by Last name, First name.
//All people returned will have valid Tnumbers, First Names, and Last names.
//
//This can throw an int on database connection error
QList<person*>* Database::getPeopleWithAccessToRooms(Database::RoomQueryMode mode, room* discover)
{
    QList<person*>* people = new QList<person*>;
    QSqlQuery query(*dbs);
    int i = 0;
    QString queryString = "select distinct TNumber,FirstName,LastName from people where ";
    if (mode == Database::BOTH || mode == Database::KEYS_ONLY)
    {
        queryString += QString("TNumber in (select distinct TNumber from p_k where TagNum in (select distinct TagNum ")+
                       QString(" from k_r where RoomID = ?))");

        i += 1;
    }
    if (mode == Database::BOTH || mode == Database::GROUPS_ONLY)
    {
        if (i == 1)
        {
            queryString += " OR ";
        }
        queryString += QString("TNumber in (select distinct TNumber from p_g  where Gid in (select distinct Gid ")+
                       QString("from g_r where RoomID = ?))");
        i += 1;
    }
    else
    {
        //shouldn't happen
        qDebug() << "Invalid mode" << endl;
    }
    query.prepare(queryString);
    qDebug() << queryString << endl;
    for (int j = 0; j < i; j++){
    qDebug() << j << endl;
        query.bindValue(j,discover->getRoomID());
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    while (query.next())
    {
        person* p = new person;
        p->setTNumber(query.value(0).toString());
        p->setFirstName(query.value(1).toString());
        p->setLastName(query.value(2).toString());
        people->append(p);
    }
    return people;

}

///////////////////////////////////////////////////////////////////////////
// End Rooms
///////////////////////////////////////////////////////////////////////////
