#include <QtGui>
#include "keywindow.h"
#include "ui_keywindow.h"
#include "database.h"
#include "checkout.h"
#include "noroom.h"

#include "key.h"
#include "room.h"
#include "person.h"


extern void setErrorLabel(QLabel*,QString);

//This function connects buttons and other signals on the key tab to functions
//It also sets restrictions on user input
void KeyWindow::keyConnections()
{
    ui->lineEdit_TagNum->setValidator(this->tagVal);
    ui->lineEdit_KeyName->setMaxLength(30);
    ui->lineEdit_KeyQuantity->setValidator(this->quantityVal);

    connect(ui->addNewKey_but,SIGNAL(clicked()),this,SLOT(addKey()));
    connect(ui->alterKeyQuantity_but,SIGNAL(clicked()),this,SLOT(changeKeyQuantity()));
    connect(ui->searchKeys_but,SIGNAL(clicked()),this,SLOT(queryKeys()));
    connect(ui->checkoutKeys_but,SIGNAL(clicked()),this,SLOT(checkout_keys()));
    connect(ui->listWidgetKeys,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(keyPrepare(QListWidgetItem*)));
    connect(ui->listWidgetKeys,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(displayKeyInfo(QListWidgetItem*)));
    connect(ui->addRoomstoKey_but,SIGNAL(clicked()),this,SLOT(modifyKeyRoomsList_add()));
    connect(ui->removeRoomsfromKey_but,SIGNAL(clicked()),this,SLOT(modifyKeyRoomsList_remove()));
    connect(ui->searchPeoplewKeys_but,SIGNAL(clicked()),this,SLOT(findPeopleWithKey()));
    connect(ui->viewAllKeys_but,SIGNAL(clicked()),this,SLOT(listKeys()));
}

//This function displays a string with information about a key when
//it's ListItem is double clicked
void KeyWindow::displayKeyInfo(QListWidgetItem* item)
{
    key k;
    k.setKeyName(item->text());
    k.setTagNum(item->toolTip().toInt());
    QList<room*>* rooms = NULL;
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
        }
        rooms = this->db->getKeyRooms(&k);
        this->db->getInventoryInfo(&k);

        QString text = getKeyInfoString(&k,rooms);

        ui->textBrowserKeys->setText(text);
        qDeleteAll(rooms->begin(),rooms->end());
        delete rooms;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 200)
        {
            setErrorLabel(ui->KeysErrorLabel,"That key could not be found.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        if (rooms != NULL)
        {
            qDeleteAll(rooms->begin(),rooms->end());
            delete rooms;
        }
    }
}

//This function takes a key name, tag number, an optional initial inventory and
//  an optional list of rooms and tries to add the key to the system with the
//  given inventory and room list
void KeyWindow::addKey()
{
    QString tagNumStr = ui->lineEdit_TagNum->text();
    if (tagNumStr.length() == 0)
    {
        setErrorLabel(ui->KeysErrorLabel,"You must give a tag number for the new key.");
        return;
    }
    //Validator insures integer
    int tagNum = tagNumStr.toInt();
    QString keyName = ui->lineEdit_KeyName->text();
    //canot
    if (keyName == "")
    {
        setErrorLabel(ui->KeysErrorLabel,"You must give a name to the new key.");
        return;
    }
    QString quantityStr = ui->lineEdit_KeyQuantity->text();
    int quantity = 1;
    if (quantityStr != "")
    {
        //again the validator ensures an integer
        quantity = quantityStr.toInt();
        if (quantity < 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"If you wish to set an initial inventory<br>it must be positive.");
            return;
        }
    }
    key k;
    k.setTable("pkeys");
    k.setTagNum(tagNum);
    k.setKeyName(keyName);
    //database function requires both of the following be set
    k.setTotal(quantity);
    k.setCurrentNumber(quantity);
    QList<room*> add;
    this->db->transaction();
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->KeysErrorLabel->setText("Database connection re-established.");
        }
        //We must add the key before we can try to add rooms to the key
        if( !this->db->addInventoryItem(&k))
        {
            setErrorLabel(ui->KeysErrorLabel,"That key could not be added.");
            this->db->rollback();
            return;
        }


        QString rooms = ui->lineEdit_KeyRoomNames->text();
        QStringList roomsList;
        if (rooms.length() != 0)
        {
            roomsList = getTrimmedRooms(rooms);
            for (int i = 0; i < roomsList.count(); i++)
            {
                room *r = new room;
                r->setRoomName(roomsList.at(i));

                this->db->doesRoomExist(r);
                //if the room is not in the system, ask the user
                if (r->getRoomID() == 0)
                {
                    noRoom n("The room named: "+*r->getRoomName()+" was not found.\nWould you like to add it?");
                    if(n.exec() == QDialog::Rejected)
                    {
                        qDeleteAll(add.begin(),add.end());
                        delete r;
                        this->db->rollback();
                        setErrorLabel(ui->KeysErrorLabel,"That key could not be added with those rooms");
                        return;
                    }
                    this->db->addRoom(r);
                }
                add.append(r);
            }

            //add the key->room associations
            this->db->modifyRoomList(Database::TO_KEY,Database::ADD_ROOMS,tagNum,&add);
            keyClear();
        }

    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 4)
        {
            setErrorLabel(ui->KeysErrorLabel,"There is already a key with that name or tag number.");
        }
        else if (i == 7)
        {
            setErrorLabel(ui->KeysErrorLabel,"Either that key or one of those rooms does not exist in the system.<br>Please check.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        qDeleteAll(add.begin(),add.end());
        this->db->rollback();
        return;
    }

    qDeleteAll(add.begin(),add.end());
    if(this->db->commit())
    {
        listKeys();
        ui->KeysErrorLabel->setText("That key was successfully added");
    }
    else
    {
        setErrorLabel(ui->KeysErrorLabel,"That key could not be added to the system.");
    }
}

//This function takes a tag number and a number to alter the keys quantity by
//  from the user. It then attempts to change the amount of that inventory in the
//  system.
void KeyWindow::changeKeyQuantity()
{
    QString tagNumStr = ui->lineEdit_TagNum->text();
    if (tagNumStr.length() == 0)
    {
        setErrorLabel(ui->KeysErrorLabel,"You must give a tag number for the new key.");
        return;
    }
    //the validator ensures an integer
    int tagNum = tagNumStr.toInt();
    QString quantityStr = ui->lineEdit_KeyQuantity->text();
    int quantity = 0;
    if (quantityStr == "")
    {
        setErrorLabel(ui->KeysErrorLabel,"You must give an amount to alter the inventory by.");
        return;
    }
    //again the validator ensures an integer
    quantity = quantityStr.toInt();
    key k;
    k.setTable("pkeys");
    k.setTagNum(tagNum);
    k.setCurrentNumber(quantity);
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
        }
        if (this->db->updateItemQuantity(&k))
        {
            ui->KeysErrorLabel->setText("That quantity was changed.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"That quantity could not be changed.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 200)
        {
            setErrorLabel(ui->KeysErrorLabel,"That item could not be found.");
        }
        else if (i == 300)
        {
            setErrorLabel(ui->KeysErrorLabel,"There was not enough of that item available to remove.<br>Please check-in some to remove more.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        return;
    }
    if (k.getAmountAvailable() == 0)
    {
        //This will remove the item from the ListWidget if it is in the list
        for (int i = 0; i < ui->listWidgetKeys->count(); i++)
        {
            QListWidgetItem * item = ui->listWidgetKeys->item(i);
            if (item->toolTip().toInt() == tagNum)
            {
                delete item;
                break;
            }
        }
    }
}

//This function empties all user input fields
void KeyWindow::keyClear()
{
    ui->lineEdit_TagNum->clear();
    ui->lineEdit_KeyName->clear();
    ui->lineEdit_KeyRoomNames->clear();
    ui->lineEdit_KeyQuantity->clear();
}

//This sets the user input fields for Tag Number and
//  Key Name for the item that is selected
void KeyWindow::keyPrepare(QListWidgetItem* item)
{
    ui->lineEdit_TagNum->setText(item->toolTip());
    ui->lineEdit_KeyName->setText(item->text());
}

//This function displays all keys in the system in the ListWidget on
//  the Keys tab
void KeyWindow::listKeys()
{
    try{
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->KeysErrorLabel->setText("Database connection re-established.");

        }
        QList<key*>* keys = this->db->getAllKeys();
        ui->listWidgetKeys->clear();
        if (keys->count() == 0){
            delete keys;
            ui->KeysErrorLabel->setText("No keys are in the system.");
            keyClear();
            return;
        }
        for (int i = 0; i < keys->count(); i++){
            key* display = keys->at(i);

            QListWidgetItem *item = new QListWidgetItem(*display->getKeyName(),ui->listWidgetKeys);
            QString TagNum;
            TagNum = TagNum.setNum(display->getTagNum());
            item->setToolTip(TagNum);
            ui->listWidgetKeys->addItem(item);

        }
        qDeleteAll(keys->begin(),keys->end());
        delete keys;
        keyClear();
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        return;
    }
}

//This function will take an optional (partial) Tag Number, an optional
//  (partial) Key Name, and an Optional list of rooms. ONE OF THESE MUST BE GIVEN.
//It attempts to find all keys in the system that match the given arguments
void KeyWindow::queryKeys()
{
    key search;
    search.setTable("pkeys");
    bool empty = true;

    //If it is there, use it
    QString tagNum = ui->lineEdit_TagNum->text();
    if (tagNum.length() != 0)
    {
        search.setTagNum(tagNum.toInt());
        empty = false;
    }

    //again use it if it is there
    QString keyName = ui->lineEdit_KeyName->text();
    if (keyName != 0)
    {
        search.setKeyName(keyName);
        empty = false;
    }

    if (empty)
    {
        setErrorLabel(ui->KeysErrorLabel,"You must give at least one paramater to search for.");
        return;
    }

    QList<room*> add;
    QList<key*>* keys = NULL;
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->KeysErrorLabel->setText("Database connection re-established.");

        }

        QString rooms = ui->lineEdit_KeyRoomNames->text();
        QStringList roomsList;
        //Process the rooms given
        if (rooms.length() != 0)
        {
            roomsList = getTrimmedRooms(rooms);
            for (int i = 0; i < roomsList.count(); i++)
            {
                room *r = new room;
                r->setRoomName(roomsList.at(i));

                this->db->doesRoomExist(r);
                if (r->getRoomID() == 0)
                {
                    setErrorLabel(ui->KeysErrorLabel,"At least one of those rooms does not exist,<br>no rooms will be found.");
                    qDeleteAll(add.begin(),add.end());
                    delete r;
                    return;
                }
                add.append(r);
            }
        }

        keys = this->db->getKeys(&search,&add);
        ui->listWidgetKeys->clear();
        if (keys->count() == 0){
            delete keys;
            ui->KeysErrorLabel->setText("No keys were found with those options.");
            keyClear();
            return;
        }
        for (int i = 0; i < keys->count(); i++){
            key* display = keys->at(i);

            QListWidgetItem *item = new QListWidgetItem(*display->getKeyName(),ui->listWidgetKeys);
            QString TagNum;
            TagNum = TagNum.setNum(display->getTagNum());
            item->setToolTip(TagNum);
            ui->listWidgetKeys->addItem(item);

        }
        qDeleteAll(keys->begin(),keys->end());
        delete keys;
        keyClear();
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        if (keys != NULL)
        {
            qDeleteAll(keys->begin(),keys->end());
            delete keys;
        }
        return;
    }
}

//This function takes a selection of items from the key tab ListWidget
//  or a Tag Number from the line Edit and tries to select these item(s)
//  for check-out
void KeyWindow::checkout_keys()
{
    QList<QListWidgetItem*> items = ui->listWidgetKeys->selectedItems();
    QList<inventory*>* keys = new QList<inventory*>;
    QString tagNumStr = ui->lineEdit_TagNum->text();
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
        }
        //If at least one item is selected, go through them and add them to
        //  the instance variable to hold the items
        if (items.count() > 0)
        {
            for (int i = 0; i < items.count(); i++)
            {
                key* k = new key;
                k->setTable("pkeys");
                k->setTagNum(items.at(i)->toolTip().toInt());
                this->db->getInventoryInfo(k);
                if (k->getAmountAvailable() != 0)
                {
                    keys->append(k);
                }
                else
                {
                    setErrorLabel(ui->KeysErrorLabel,"At least one of those keys was not<br>included because of non-availibility.");
                }
            }
        }
        else
        {//Otherwise, look into the lineEdit for the Tag Number
            if (tagNumStr.length() == 0)
            {
                delete keys;
                setErrorLabel(ui->KeysErrorLabel,"You must give a tag number for the key.");
                return;
            }
            int tagNum = tagNumStr.toInt();
            key* item = new key;
            item->setTable("pkeys");
            item->setTagNum(tagNum);
            this->db->getInventoryInfo(item);
            if (item->getAmountAvailable() != 0)
            {
                keys->append(item);
            }
            else
            {
                setErrorLabel(ui->KeysErrorLabel,"At least one of those keys was not<br>included because of non-availibility.");
            }
        }

    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 200)
        {
            setErrorLabel(ui->KeysErrorLabel,"One of those keys could not be found.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        qDeleteAll(keys->begin(),keys->end());
        delete keys;
        return;
    }
    //see if we can start a checkout procedure
    //due to precautions I have taken in keywindow.cpp with regards to enabling
    //and disabling buttons this should never happen
    if (!setPath(KeyWindow::CHECK_OUT))
    {
        qDeleteAll(keys->begin(),keys->end());
        delete keys;
        setErrorLabel(ui->KeysErrorLabel,QString("You are in the process of either checking out an item or adding a person to a group already.\n")+
                      QString("Please either cancel or complete that before adding a person to a group."));
        return;
    }
    if (keys->count() == 0)
    {
        qDeleteAll(keys->begin(),keys->end());
        delete keys;
        setErrorLabel(ui->KeysErrorLabel,"None of those keys had any available<br>to check-out.");
        pathEnd();
        return;
    }
    this->inv = keys;
    //This dialog allows the user to select the amount of each item to check-out
    //  as well as whether a due date is requested for these items
    Checkout ch(inv,this);
    if (ch.exec() == QDialog::Rejected)
    {
        this->pathEnd();
        setErrorLabel(ui->KeysErrorLabel,"The check-out was canceled.");
        return;
    }
    else
    {//Continue on to the people screen to select a person
        ui->PersonErrorLabel->setText("Please select a person to check-out the inventory.");
        ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
    }
}

//This function takes a tag number and attempts to locate and display all
//  people who have a copy of this key checked-out
void KeyWindow::findPeopleWithKey()
{
    try{
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");
        }
        QString tagNum = ui->lineEdit_TagNum->text();
        if (tagNum == ""){
            setErrorLabel(ui->KeysErrorLabel,"You must select a key to search with.");
            return;
        }
        key k;
        k.setTable("pkeys");
        //validator ensures an integer
        k.setTagNum(tagNum.toInt());
        this->db->getInventoryInfo(&k);

        QList<person*>* people = this->db->getPeopleWithInventoryCheckedOut(&k);

        ui->listWidgetPeople->clear();
        ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
        if (people->count() == 0){
            delete people;
            ui->PersonErrorLabel->setText("No people found with that query.");
            personClear();
            return;
        }
        for (int i = 0; i < people->count(); i++){
            person* display = people->at(i);

            QListWidgetItem *item = new QListWidgetItem((*display->getLastName())
                                    +", "+(*display->getFirstName()),ui->listWidgetPeople);
            item->setToolTip(*display->getTNumber());
            ui->listWidgetPeople->addItem(item);

        }
        qDeleteAll(people->begin(),people->end());
        delete people;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 200)
        {
            setErrorLabel(ui->KeysErrorLabel,"That key could not be found in the system.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
    }
}

//This takes a tag number and a comma separated list of
//room names and attempts to make a connection between the
//key and each room
void KeyWindow::modifyKeyRoomsList_add()
{
    QString tagNum = ui->lineEdit_TagNum->text();
    QString rooms = ui->lineEdit_KeyRoomNames->text();
    //neither may be empty
    if (tagNum == ""){
        setErrorLabel(ui->KeysErrorLabel,"You must select a key.");
        return;
    }
    if (rooms == ""){
        setErrorLabel(ui->KeysErrorLabel,"You must give at least one room name to try to add rooms.");
        return;
    }
    QList<room*> add;

    try {
        this->db->transaction();
        QStringList roomsList;


        roomsList = getTrimmedRooms(rooms);
        for (int i = 0; i < roomsList.count(); i++){
            room *r = new room;
            r->setRoomName(roomsList.at(i));

            this->db->doesRoomExist(r);
            if (r->getRoomID() == 0)
            {//ask the user about rooms that do not exist
                noRoom n("The room named: "+*r->getRoomName()+" was not found.\nWould you like to add it?");
                if(n.exec() == QDialog::Rejected)
                {
                    this->db->rollback();
                    setErrorLabel(ui->KeysErrorLabel,"Those rooms could not be added to the key.");
                    return;
                }
                this->db->addRoom(r);
            }
            add.append(r);
        }


        if (!this->db->modifyRoomList(Database::TO_KEY,Database::ADD_ROOMS,tagNum.toInt(),&add))
        {
            this->db->rollback();
            setErrorLabel(ui->KeysErrorLabel,"Those rooms could not be added to the key.");
            qDeleteAll(add.begin(),add.end());
            return;
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 4)
        {
            setErrorLabel(ui->KeysErrorLabel,"There is already a key with that name or tag number.");
        }
        else if (i == 7)
        {
            setErrorLabel(ui->KeysErrorLabel,"That key does not exist in the system.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        qDeleteAll(add.begin(),add.end());
        this->db->rollback();
        return;
    }

    if(this->db->commit()){
        ui->KeysErrorLabel->setText("Those rooms were successfully added to the group.");
    }
    else {
        setErrorLabel(ui->KeysErrorLabel,"Those rooms couldn't be added to the group.");
    }
    qDeleteAll(add.begin(),add.end());
}

//This function takes a tag number and a comma seperated room list
//  and tries to dis-associate the rooms from the key
void KeyWindow::modifyKeyRoomsList_remove()
{
    QString tagNum = ui->lineEdit_TagNum->text();
    QString rooms = ui->lineEdit_KeyRoomNames->text();
    //neither can be empty
    if (tagNum == "")
    {
        setErrorLabel(ui->KeysErrorLabel,"You must select a key.");
        return;
    }
    if (rooms == "")
    {
        setErrorLabel(ui->KeysErrorLabel,"You must give at least one room name to try to remove rooms.");
        return;
    }
    QList<room*> add;
    try
    {
        this->db->transaction();
        QStringList roomsList;


        roomsList = getTrimmedRooms(rooms);
        for (int i = 0; i < roomsList.count(); i++)
        {
            room *r = new room;
            r->setRoomName(roomsList.at(i));
            //if the room doesn't exist, don't bother trying to remove it
            this->db->doesRoomExist(r);
            if (r->getRoomID() != 0)
            {
                add.append(r);
            }
        }


        if(!this->db->modifyRoomList(Database::TO_KEY,Database::REMOVE_ROOMS,tagNum.toInt(),&add))
        {
            this->db->rollback();
            setErrorLabel(ui->RoomErrorLabel,"Those rooms could not be removed from the key.");
            qDeleteAll(add.begin(),add.end());
            return;
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->KeysErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 7)
        {
            setErrorLabel(ui->KeysErrorLabel,"That key does not exist in the system.");
        }
        else
        {
            setErrorLabel(ui->KeysErrorLabel,"There was an unknown error.");
        }
        qDeleteAll(add.begin(),add.end());
        this->db->rollback();
        return;
    }

    if(this->db->commit())
    {
        ui->KeysErrorLabel->setText("Those rooms were successfully removed from the key.");
    }
    else
    {
        setErrorLabel(ui->KeysErrorLabel,"Those rooms couldn't be removed from the key.");
    }
    qDeleteAll(add.begin(),add.end());
}

//This function generates the string displaying information about the key
//  whose item was double clicked in the list widget
QString KeyWindow::getKeyInfoString(key* k, QList<room*>* rooms)
{
    QString info;
    info += *k->getKeyName()+"\n";
    QString num;
    num = num.setNum(k->getTagNum());
    info += "Tag Number: "+num+"\n";

    int i = 0;
    info += "\n-Total Number-\t-Number Available-\n";
    num = num.setNum(k->getTotal());
    info += num+"\t";
    num = num.setNum(k->getAmountAvailable());
    info += num+"\n";

    QString roomString = "\n--Rooms--\n";
    for (; i < rooms->count(); i++)
    {
        room* display = rooms->at(i);
        roomString += *display->getRoomName()+"\n";
    }
    if (i == 0)
    {
        info += "\nNo rooms are opened by this key.\n";
    }
    else
    {
        info += roomString;
    }
    return info;
    return info;
}
